@@ -9,19 +9,28 @@ from pathlib import Path
 from py4ami.util import SScript
 
 # HTML
+H_HTML = "html"
+H_HEAD = "head"
+H_META = "meta"
+H_STYLE = "style"
+H_BODY = "body"
 H_TABLE = "table"
 H_THEAD = "thead"
 H_TBODY = "tbody"
 H_TR = "tr"
 H_TH = "th"
 H_TD = "td"
-H_BODY = "body"
 H_DIV = "div"
 H_SPAN = "span"
+H_UL = "ul"
+H_LI = "li"
 H_A = "a"
 H_B = "b"
 H_P = "p"
-H_HREF = "href"
+
+A_HREF = "href"
+A_NAME = "name"
+A_TITLE = "title"
 A_ID = "id"
 A_CLASS = "class"
 
@@ -50,11 +59,14 @@ STYLES = [
     STROKE,
 ]
 
+
 class AmiSpan:
     def __init__(self):
         self.text_style = None
         self.string = ""
         self.xx = []
+        self.x0 = None
+        self.x1 = None
         self.y0 = None
         # self.adv = None
 
@@ -66,6 +78,11 @@ class AmiSpan:
             html_span.attrib["style"] = self.text_style.create_css_string()
             if len(self.xx) > 0:
                 html_span.attrib["x"] = self.xx[0]
+            if self.x0:
+                # print(f"x0 {self.x0}")
+                html_span.attrib["x0"] = str(self.x0)
+            if self.x1:
+                html_span.attrib["x1"] = str(self.x1)
             html_span.attrib["y"] = str(self.y0)
         return html_span
 
@@ -82,7 +99,7 @@ class HtmlUtil:
 
     @classmethod
     def split_span_at_match(cls, elemx, regex, copy_atts=True, recurse=True, id_root=None, id_counter=0,
-                            new_tags=[H_SPAN, H_SPAN, H_SPAN]):
+                            new_tags=None):
         """splits an elem (normally span) into 3 components by regex match
         :param elemx: elem to split (normally has a parent (e.g. div)
         :param regex: regex to split elem (of form (pre)(match)(post)
@@ -93,12 +110,15 @@ class HtmlUtil:
         :param new_tags: new_element tags (default span, span, span)
         :return: list of 3 elems; if new_elems[2] is not None it's available for recursion)
         """
+        if not new_tags:
+            new_tags = [H_SPAN, H_SPAN, H_SPAN]
         assert elemx is not None
         textx = ''.join(elemx.itertext())
         rec = re.compile(regex)
         match = rec.match(textx)
         new_elems = [None, None, None]
         if match:
+            parent = elemx.getparent()
             assert len(match.groups()) == 3  # some may be empty strings
             group1 = match.group(1)
             if group1 != "":  # don't add empty string
@@ -108,6 +128,7 @@ class HtmlUtil:
                 new_elems[0].attrib["class"] = "re_pref"
                 id_counter = cls.add_id_increment_counter(id_counter, id_root, elemx)
             new_elems[1] = HtmlUtil.add_sibling_after(elemx, new_tags[1], copy_atts=copy_atts, text=match.group(2))
+            # print(f" match 2 {match.group(2)}")
             new_elems[1].attrib["class"] = "re_match"
             id_counter = cls.add_id_increment_counter(id_counter, id_root, new_elems[1])
             if match.group(3) != "":  # don't add empty string
@@ -132,7 +153,7 @@ class HtmlUtil:
     @classmethod
     def add_sibling_after(cls, anchor_elem, tag, replace=False, copy_atts=False, text=None):
         """adds new trailing sibling of anchor_elem with tag
-        :param tag: tag for new elements
+        :param tag: tag for new element
         :param anchor_elem: reference element, must have a parent
         :param replace: if True, remove anchor element
         :param copy_atts: copy attributes from anchor
@@ -160,7 +181,7 @@ class HtmlUtil:
 
     @classmethod
     def create_div_span(cls, text, style=None):
-        """utilitymethodto create a div/span@text (probably mainly for testing)
+        """utility method to create a div/span@text (probably mainly for testing)
         :param text: to add
         :return: the div"""
         div = lxml.etree.Element(H_DIV)
@@ -190,12 +211,13 @@ class HtmlUtil:
         return cls.is_script_type(last_span, this_span, script_type=SScript.SUP)
 
     @classmethod
-    def is_script_type(cls, last_span, this_span, script_type) -> bool:
+    def is_script_type(cls, last_span, this_span, script_type, ydown=True) -> bool:
         """heuristc to determine whether this_span is a sub/superscript of last_span
-        NOTE: as Y is DOWN the page, a superscript has SMALLER y-value, etc.
+        NOTE: IF Y is DOWN the page, a superscript has SMALLER y-value, etc. (logic reversed if not ydown)
         :param last_span: if None, returns false
         :param this_span: if not smaller by SCRIPT_FACT return False
-        :param script_type: SUB or SUP
+        :param script_type: SScript.SUB or SScript.SUP
+        :param ydown: True if y increases down thr page (e.g. SVG) (DEFAULT) else False
         :return: True if smaller and moved in right y-direction
         """
         if last_span is None:
@@ -208,10 +230,10 @@ class HtmlUtil:
             this_y = this_span.y
             if script_type == SScript.SUB:
                 # is it lowered? Y DOWN
-                return last_y < this_y
+                return ydown and (last_y < this_y)
             elif script_type == SScript.SUP:
                 # is it raised? Y DOWN
-                return last_y > this_y
+                return ydown and (last_y > this_y)
             else:
                 raise ValueError("bad script type ", script_type)
         else:
@@ -239,6 +261,36 @@ class HtmlUtil:
         for i, el in enumerate(elems):
             el.attrib[A_ID] = A_ID + str(i)
 
+    def join_spans_in_div(self, html):
+        """joint <span>...</span><span>...</span> into <span>...</span> recursively
+        this structure arises when PDF or images is parsed and spans have the same styles (size/style/weight) and can be merged
+        :param div: contains the sibling spans
+        """
+        print(f" NYI")
+
+    @classmethod
+    def create_skeleton_html(cls):
+        """create empty html tree
+        html
+            head
+                meta
+                # style
+            body
+
+        :return: this html
+        """
+
+        html = lxml.etree.Element(H_HTML)
+        head = lxml.etree.Element(H_HEAD)
+        html.append(head)
+        meta = lxml.etree.Element(H_META)
+        head.append(meta)
+        style = lxml.etree.Element(H_STYLE)  # empty <style> means no display
+        # head.append(style)
+        body = lxml.etree.Element(H_BODY)
+        html.append(body)
+        return html
+
 
 class HtmlTree:
     """builds a tree from a flat set of Html elemnets"""
@@ -261,8 +313,10 @@ class HtmlTree:
     ALL_DIV_XPATHS = ".//div"
 
     @classmethod
-    def make_tree(cls, elem, output_dir, recs_by_section=None):
-        """find decimal number for tree"""
+    def make_sections_and_output(cls, elem, output_dir, recs_by_section=None):
+        """find decimal number for tree
+        ThiS is HEAVILY based on IPCC
+        """
         markers = ["Chapter",
                    # "Table of Contents",
                    "Table of",  # one is concatenated (Chapter01) so abbreviate, unweighted Chap16
@@ -276,7 +330,7 @@ class HtmlTree:
         font_size_range = (12, 999)
         for marker in markers:
             marked_div, divs = cls.get_div_span_starting_with(elem, marker, is_bold, font_size_range=font_size_range)
-            if not marked_div:
+            if marked_div is not None:
                 ld = len(divs) if divs else 0
                 print(f"Cannot find marker {marker} found {ld} markers")
 
@@ -293,10 +347,12 @@ class HtmlTree:
             if not output_dir.exists():
                 output_dir.mkdir()
             for i, child_div in enumerate(decimal_divs):
-                marker = child_div.attrib[HtmlUtil.MARKER].strip().replace(" ", "_").lower()  # name from text content
-                path = Path(output_dir, f"{marker}.html")
-                with open(path, "wb") as f:
-                    f.write(lxml.etree.tostring(child_div, pretty_print=True))
+                if HtmlUtil.MARKER in child_div.attrib:
+                    marker = child_div.attrib[HtmlUtil.MARKER].strip().replace(" ",
+                                                                               "_").lower()  # name from text content
+                    path = Path(output_dir, f"{marker}.html")
+                    with open(path, "wb") as f:
+                        f.write(lxml.etree.tostring(child_div, pretty_print=True))
             print(f"decimals: {len(decimal_divs)}")
 
     @classmethod
@@ -358,7 +414,17 @@ class HtmlTree:
             spans = div.xpath("./span")
             if not spans:
                 # no spans, concatenate with siblings
-                current_div.append(div)
+                if div == current_div:
+                    print("f cannot append div {div} to itself")
+                else:
+                    try:
+                        current_div.append(div)
+                    except ValueError as ve:
+                        print(f"Error {ve}")
+                try:
+                    current_div.append(div)
+                except Exception as e:
+                    print(f"BUG skipped")
                 continue
             css_style = CSSStyle.create_css_style(spans[0])  # normally comes first
             # check weight, if none append to siblings
@@ -583,17 +649,16 @@ class CSSStyle:
         rgb_scale = 255
         # cmyk_scale = 100
         cmyk_scale = 1.0
-        r = rgb_scale*(1.0-(c+k)/float(cmyk_scale))
-        g = rgb_scale*(1.0-(m+k)/float(cmyk_scale))
-        b = rgb_scale*(1.0-(y+k)/float(cmyk_scale))
+        r = rgb_scale * (1.0 - (c + k) / float(cmyk_scale))
+        g = rgb_scale * (1.0 - (m + k) / float(cmyk_scale))
+        b = rgb_scale * (1.0 - (y + k) / float(cmyk_scale))
 
-        return r,g,b
+        return r, g, b
 
     @classmethod
     def cmky_to_rgb(cls, c, m, k, y):
         return cls.cmyk_to_rgb(c, m, y, k)
 
-
     def extract_bold_italic_from_font_family(self, overwrite_bold=False, overwrite_style=False, overwrite_family=True,
                                              style_regex=STYLE_RE, weight_regex=WEIGHT_RE):
         """heuristics to find bold and italic in font names and try to normalise
@@ -624,6 +689,3 @@ class CSSStyle:
         else:
             value = None
         return family, value
-
-
-
