@@ -1,6 +1,9 @@
 """ Mainly for converting PDF to HTML and SVG """
 import argparse
+import logging
 import os.path
+import sys
+import copy
 
 import lxml
 import lxml.html
@@ -19,6 +22,7 @@ from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
 from pdfminer.pdfpage import PDFPage
 from pdfminer.image import ImageWriter
 from pdfminer.layout import LTImage
+import pdfplumber
 from PIL import Image
 
 from sklearn.linear_model import LinearRegression
@@ -26,10 +30,10 @@ from sklearn.linear_model import LinearRegression
 # local
 from py4ami.bbox_copy import BBox  # this is horrid, but I don't have a library
 from py4ami.util import Util, AbstractArgs
-from py4ami.ami_html import HtmlUtil, CSSStyle, HtmlTree
+from py4ami.ami_html import HtmlUtil, CSSStyle, HtmlTree, AmiSpan
 from py4ami.ami_html import STYLE, BOLD, ITALIC, FONT_FAMILY, FONT_SIZE, FONT_WEIGHT, FONT_STYLE, STROKE, FILL, TIMES, \
-    CALIBRI, FONT_FAMILIES
-from py4ami.ami_html import H_SPAN, H_A, H_HREF, H_TR, H_TD, H_TABLE, H_THEAD, H_TBODY
+    CALIBRI, FONT_FAMILIES, H_DIV, H_BODY
+from py4ami.ami_html import H_SPAN, H_A, A_HREF, H_TR, H_TD, H_TABLE, H_THEAD, H_TBODY
 
 # text attributes
 FACT = 2.8
@@ -76,6 +80,7 @@ P_NON_STROKING_COLOR = "non_stroking_color"
 P_X0 = "x0"
 P_X1 = "x1"
 P_Y0 = "y0"
+P_Y1 = "y1"
 P_TEXT = "text"
 
 # Unwanted sections
@@ -110,17 +115,21 @@ class AmiPage:
     consists of paragraphs, divs, textlines, etc.
     Used as a working container, utimately being merged with
     neighbouring documents into complete HTML document
+
+    Now including HTML divs and spans from PDF
+    MESSY (because PDF is horrible)
     """
     CONTENT_RANGES = [[56, 999], [45, 780]]
 
     def __init__(self):
+        # a mess because it started with SVG and new we are adding PDF
         # path of SVG page
         self.page_path = None
         # raw parsed SVG
         self.page_element = None
         # child elements of type <climate10_:text>
         self.text_elements = None
-        # spans created from tex_elements
+        # spans created from text_elements
         self.text_spans = []
         # bboxes of the spans
         self.bboxes = []
@@ -128,6 +137,24 @@ class AmiPage:
         self.composite_lines = []
         # paragraphs from inter-composite spacing
         self.paragraphs = []
+        # AmiSpans built from characters from pdf
+        self.ami_spans = []
+
+    @classmethod
+    def create_page_from_ami_spans_from_pdf(cls, ami_spans, bboxes=None):
+        """
+        create from raw AmiSpans (probably created from PDF)
+        Tidying into divs, etc is done elsewhere
+        :param ami_spans: list of AmiSpans possibly in document order
+        :param bboxes: boxes within which spans must fit (if None accept everything)
+        :return: AmiPage (containing AmiSpans) or None
+        """
+        ami_page = None
+        if ami_spans:
+            ami_page = AmiPage()
+            for ami_span in ami_spans:
+                ami_page.ami_spans.append(copy.deepcopy(ami_span))
+
 
     @classmethod
     def create_page_from_svg(cls, svg_path, rotated_text=False):
@@ -217,6 +244,8 @@ class AmiPage:
                 self.bboxes.append(bbox)
         return self.bboxes
 
+    # AmiPage
+
     def create_composite_lines(self) -> list:
         """overlaps textspans such as subscripts
         uses the bboxes
@@ -225,12 +254,15 @@ class AmiPage:
         self.composite_lines = []
         self.create_text_spans(sort_axes=SORT_XY)
         if not self.text_spans:
-            return []
+            return self.composite_lines
+
+        return self.create_composite_lines_from_text_spans()
+
+    def create_composite_lines_from_text_spans(self):
         span0 = self.text_spans[0]
         composite_line = CompositeLine(bbox=span0.bbox)
         composite_line.text_spans.append(span0)
         self.composite_lines.append(composite_line)
-
         for text_span in self.text_spans[1:]:
             bbox = text_span.create_bbox().copy()
             bbox.expand_by_margin([X_MARGIN, 0])
@@ -243,11 +275,9 @@ class AmiPage:
                 composite_line.bbox = bbox.copy()
 
             composite_line.text_spans.append(text_span)
-
         change = True
         while change:
             change = self.merge_composite_lines()
-
         return self.composite_lines
 
     def merge_composite_lines(self):
@@ -390,6 +420,97 @@ class AmiPage:
             et = lxml.etree.ElementTree(html)
             et.write(f, pretty_print=pretty_print)
 
+    # AmiPage
+
+    @classmethod
+    # TODO should be new class
+    def chars_to_spans(cls, bbox, input_pdf, page_no):
+        with pdfplumber.open(input_pdf) as pdf:
+            pdf_page = pdf.pages[page_no]
+            ami_page = AmiPage()
+            # print(f"crop: {page0.cropbox} media {page0.mediabox}, bbox {page0.bbox}")
+            # print(f"rotation: {page0.rotation} doctop {page0.initial_doctop}")
+            # print(f"width {page0.width} height {page0.height}")
+            # print(f"text {page0.extract_text()[:2]}")
+            # print(f"words {page0.extract_words()[:3]}")
+            #
+            # print(f"char {page0.chars[:1]}")
+            span = None
+            span_list = []
+            maxchars = 999999
+            ndec_coord = 3  # decimals for coords
+            ndec_fontsize = 2
+            html = HtmlUtil.create_skeleton_html()
+            top_div = lxml.etree.SubElement(html.xpath(H_BODY)[0], H_DIV)
+            top_div.attrib["class"] = "top"
+            for ch in pdf_page.chars[:maxchars]:
+                if cls.skip_rotated_text(ch):
+                    continue
+                x0, x1, y0, y1 = cls.get_xy_tuple(ch, ndec_coord)
+                if bbox and not bbox.contains_point((x0, y0)):
+                    # print(f" outside box: {x0, y0}")
+                    continue
+
+                text_style = TextStyle()
+                text_style.set_font_family(ch.get(P_FONTNAME))
+                text_style.set_font_size(ch.get(P_HEIGHT), ndec=ndec_fontsize)
+                text_style.stroke = ch.get(P_STROKING_COLOR)
+                text_style.fill = ch.get(P_NON_STROKING_COLOR)
+
+                # style or y0 changes
+                if not span or not span.text_style or span.text_style != text_style or span.y0 != y0:
+                    # cls.debug_span_changed(span, text_style, y0)
+                    span = AmiSpan()
+                    span_list.append(span)
+                    span.text_style = text_style
+                    span.y0 = y0
+                    span.x0 = x0  # set left x
+                span.x1 = x1  # update right x, including width
+                span.string += ch.get(P_TEXT)
+
+            # top_div = lxml.etree.Element(H_DIV)
+            div = lxml.etree.SubElement(top_div, H_DIV)
+            last_span = None
+            for span in span_list:
+                if last_span is None or last_span.y0 != span.y0:
+                    div = lxml.etree.SubElement(top_div, H_DIV)
+                last_span = span
+                span.create_and_add_to(div)
+        for ch in pdf_page.chars[:maxchars]:
+            col = ch.get('non_stroking_color')
+            if col:
+                print(f"txt {ch.get('text')} : col {col}")
+        # print(f"HTML {html}")
+        return html
+
+    @classmethod
+    def debug_span_changed(cls, span, text_style, y0):
+        if span:
+            if span.text_style != text_style:
+                print(f"{span.text_style.difference(text_style)} \n {span.string}")
+
+            if span.y0 != y0:
+                print(f""
+                      f"Y {y0} != {span.y0}\n {span.string} {span.xx} ")
+
+    @classmethod
+    def get_xy_tuple(cls, ch, ndec_coord):
+        x0 = round(ch.get(P_X0), ndec_coord)
+        x1 = round(ch.get(P_X1), ndec_coord)
+        y0 = round(ch.get(P_Y0), ndec_coord)
+        y1 = round(ch.get(P_Y1), ndec_coord)
+        return x0, x1, y0, y1
+
+    @classmethod
+    def skip_rotated_text(cls, ch):
+        """is text rotated? uses matrix"""
+        matrix = ch.get("matrix")
+        return matrix and matrix[0:4] != (1, 0, 0, 1)
+
+    @classmethod
+    def create_page_from_pdf_html(cls, path):
+        logging.error("NOT YET WRITTEN")
+
 
 class AmiSect:
     """Transformation of an Html Page to sections
@@ -557,13 +678,21 @@ class TextSpan:
 
 
 # arg_dict
-MAXPAGE = "maxpage"
+DEFAULT_MAXPAGES = 100
+DEFAULT_CONVERT = "html"
+
+CONVERT = "convert"
+FLOW = "flow"
+FOOTER = "footer"
+HEADER = "header"
 INDIR = "indir"
 INPATH = "inpath"
+MAXPAGE = "maxpage"
 OUTDIR = "outdir"
+OUTPATH = "outpath"
 OUTFORM = "outform"
 OUTSTEM = "outstem"
-FLOW = "flow"
+
 # FORMAT = "fmt"
 IMAGEDIR = "imagedir"
 RESOLUTION = "resolution"
@@ -576,7 +705,20 @@ class PDFArgs(AbstractArgs):
     def __init__(self):
         """arg_dict is set to default"""
         super().__init__()
+        self.convert = DEFAULT_CONVERT
+        self.out_fmt = DEFAULT_CONVERT
+        self.html = None
+        self.indir = None
+        self.inpath = None
+        self.maxpage = DEFAULT_MAXPAGES
+        self.outdir = None
+        self.outpath = None
+        self.outstem = None
+        self.raw_html = None
+        self.flow = None
+        self.unwanteds = None
 
+    @property
     def module_stem(self):
         return "ami_pdf"
 
@@ -584,18 +726,24 @@ class PDFArgs(AbstractArgs):
         """creates adds the arguments for pyami commandline
 
         """
+        print(f"sys.argv {sys.argv}")
         self.parser = argparse.ArgumentParser(description='PDF parsing')
-        self.parser.add_argument("--maxpage", type=int, nargs=1, help="maximum number of pages", default=10)
-        self.parser.add_argument("--indir", type=str, nargs=1, help="input directory")
-        self.parser.add_argument("--inpath", type=str, nargs=1, help="input file")
-        self.parser.add_argument("--outdir", type=str, nargs=1, help="output directory")
-        self.parser.add_argument("--outform", type=str, nargs=1, help="output format ", default="html")
+        self.parser.add_argument("--convert", type=str, choices=[], help="conversions (NYI)")
+        self.parser.add_argument("--debug", type=str, choices=DEBUG_OPTIONS, help="debug these during parsing (NYI)")
         self.parser.add_argument("--flow", type=bool, nargs=1, help="create flowing HTML (heuristics)", default=True)
+        self.parser.add_argument("--footer", type=float, nargs=1, help="top margin (clip everythimg above)", default=80)
+        self.parser.add_argument("--header", type=float, nargs=1, help="bottom margin (clip everything below", default=80)
         self.parser.add_argument("--imagedir", type=str, nargs=1, help="output images to imagedir")
+        self.parser.add_argument("--indir", type=str, nargs=1, help="input directory")
+        self.parser.add_argument("--inpath", type=str, nargs=1, help="input file or (NYI) url")
+        self.parser.add_argument("--maxpage", type=int, nargs=1, help="maximum number of pages (will be deprecated, use 'pages')", default=self.arg_dict.get(MAXPAGE))
+        self.parser.add_argument("--pages", type=tuple, nargs="+", help="NYI (reads (1,3) or [(1, 3), (5, 10)]", default=[])
+        self.parser.add_argument("--outdir", type=str, nargs=1, help="output directory")
+        self.parser.add_argument("--outstem", type=str, nargs=1, help="output file", default="fulltext.flow")
+        self.parser.add_argument("--outform", type=str, nargs=1, help="output format ", default="html")
         self.parser.add_argument("--resolution", type=int, nargs=1, help="resolution of output images (if imagedir)",
                                  default=400)
         self.parser.add_argument("--template", type=str, nargs=1, help="file to parse specific type of document (NYI)")
-        self.parser.add_argument("--debug", type=str, choices=DEBUG_OPTIONS, help="debug these during parsing (NYI)")
         return self.parser
 
     # class PDFArgs:
@@ -615,21 +763,77 @@ class PDFArgs(AbstractArgs):
   --template TEMPLATE   file to parse specific type of document"""
 
         if self.arg_dict:
-            fmt = self.arg_dict.get(OUTFORM)
-            print(f"fmt: {fmt}")
-            maxpage = self.arg_dict.get(MAXPAGE)
-            indir = self.arg_dict.get(INDIR)
-            inpath = self.arg_dict.get(INPATH)
-            outdir = self.arg_dict.get(OUTDIR)
-            outstem = self.arg_dict.get(OUTSTEM)
-            flow = self.arg_dict.get(FLOW) is not None
-            if not inpath:
-                print(f"input file not given")
+            self.read_arg_dict()
+
+        if not self.check_input():
+            print(f"no input given")
+            return
+        self.check_output()
+        self.calculate_headers_footers()
+
+        # self.convert_write()
+
+    def check_input(self):
+        if not self.inpath:
+            print(f"No input file, no action taken")
+            return False
+            # raise FileNotFoundError(f"input file not given")
+        if not Path(self.inpath).exists():
+            raise FileNotFoundError(f"input file does not exist: ({self.inpath}")
+        self.indir = Path(self.inpath).parent
+        return True
+
+    def check_output(self):
+        print(f" check_output {self.arg_dict}")
+        self.arg_dict[OUTSTEM] = Path(f"{self.inpath}").stem
+        self.arg_dict[OUTPATH] = Path(Path(self.inpath).parent, f"{self.arg_dict[OUTSTEM]}.{self.arg_dict[OUTFORM]}")
+
+        if not self.outpath:
+            self.outpath = self.arg_dict.get(OUTPATH)
+        if self.outpath:
+            self.outdir = Path(self.outpath).parent
+        elif not self.outdir:
+            if self.inpath:
+                self.outdir = Path(self.inpath).parent
             else:
-                inpath = Path(inpath)
-                if not inpath.exists():
-                    raise FileNotFoundError(f"input file does not exist: ({inpath}")
-                self.convert_write(maxpage=maxpage, outdir=outdir, outstem=outstem, fmt=fmt, inpath=inpath, flow=True)
+                raise FileNotFoundError(f"output dir not given and cannot be generated")
+            self.outstem = self.arg_dict.get(OUTSTEM)
+            if not self.outstem:
+                raise FileNotFoundError(f"output file not given and cannot be generated")
+            if not self.out_fmt:
+                raise FileNotFoundError(f"no output format given")
+            self.outpath = Path(self.outdir, f"{self.outstem}.{self.out_fmt}")
+            print(f"outpath {self.outpath}")
+        else:
+            print(f"outdir {self.outdir}")
+        if not Path(self.outdir).exists():
+            Path(self.outdir).mkdir()
+        elif not Path(self.outdir).is_dir():
+            raise ValueError(f"output dir {self.outdir} is not a directory")
+        else:
+            print(f"output dir {self.outdir}")
+        return True
+
+
+
+
+    def read_arg_dict(self):
+        self.flow = self.arg_dict.get(FLOW) is not None
+        self.footer = self.arg_dict.get(FOOTER)
+        if not self.footer:
+            self.footer = 80
+        self.header = self.arg_dict.get(HEADER)
+        if not self.header:
+            self.header = 80
+        self.indir = self.arg_dict.get(INDIR)
+        self.inpath = self.arg_dict.get(INPATH)
+        self.maxpage = self.arg_dict.get(MAXPAGE)
+        self.outdir = self.arg_dict.get(OUTDIR)
+        self.out_fmt = self.arg_dict.get(OUTFORM)
+        self.outstem = self.arg_dict.get(OUTSTEM)
+        self.convert = self.arg_dict.get(CONVERT)
+
+            # self.convert_write(maxpage=maxpage, outdir=outdir, outstem=outstem, fmt=fmt, inpath=inpath, flow=True)
 
     # class PDFArgs:
     @classmethod
@@ -642,6 +846,7 @@ class PDFArgs(AbstractArgs):
                     caching: bool = True,
                     pagenos: Container[int] = set(),
                     ) -> str:
+        """Uses PDFMiner library (I think) which omits coordinates"""
         """Summary
         Parameters
         ----------
@@ -697,17 +902,22 @@ class PDFArgs(AbstractArgs):
     def create_default_arg_dict(cls):
         """returns a new COPY of the default dictionary"""
         arg_dict = dict()
-        arg_dict[OUTFORM] = "html.flow"
-        arg_dict[MAXPAGE] = 5
+        arg_dict[CONVERT] = "html"
+        arg_dict[FLOW] = True
+        arg_dict[FOOTER] = 80
+        arg_dict[HEADER] = 80
         arg_dict[INDIR] = None
         arg_dict[INPATH] = None
+        arg_dict[MAXPAGE] = 5
         arg_dict[OUTDIR] = None
+        arg_dict[OUTFORM] = "html"
         arg_dict[OUTSTEM] = None
         arg_dict[FLOW] = True
         return arg_dict
 
     @classmethod
     def create_pdf_interpreter(cls, fmt, codec: str = "UTF-8"):
+        """Based on PDFMiner I think"""
         """creates a PDFPageInterpreter
         :format: "text, "xml", "html"
         :codec: default UTF-8
@@ -735,99 +945,66 @@ class PDFArgs(AbstractArgs):
         converters = {"text": TextConverter, "html": HTMLConverter, "flow.html": HTMLConverter, "xml": XMLConverter}
         converter = converters.get(fmt)
         if not converter:
-            raise ValueError(f"provide format, {converters.keys()}")
+            raise ValueError(f"format ({fmt}) is invalid, {converters.keys()}")
         device = converter(rsrcmgr, retstr, codec=codec, laparams=laparams)
         interpreter = PDFPageInterpreter(rsrcmgr, device)
         return device, interpreter, retstr
 
     # class PDFArgs:
 
-    def convert_write(self, fmt=None, maxpage=999999, outdir=None, outstem=None, inpath=None, flow=False,
-                      unwanteds=None):
-        """
-        create HTML (absolute or flowing) or XML
-        The preferred method is to use arg_dict
-        :param fmt: format html/xml/text
-        :param maxpage: if 0, writes all else staops at maxpages
-        :param outdir: output dir
-        :param outstem: stem of output file
-        :param inpath: input file
-        :param flow: remove absolute position so text can flow
-        """
-        if self.arg_dict:
-            maxp = self.arg_dict.get(MAXPAGE)
-            maxpage = int(maxp) if maxp else maxpage
-            outd = self.arg_dict.get(OUTDIR)
-            outdir = outd if outd else outdir
-            if not outdir:
-                outs = self.arg_dict.get(OUTSTEM)
-                outdir = outs if outs else outstem
-            inp = self.arg_dict.get(INPATH)
-            inpath = inp if inp else inpath
-            if fm := self.arg_dict.get(OUTFORM):
-                fmt = fm
-            if fl := self.arg_dict.get(FLOW):
-                flow = fl
-
-            # header_offset = -50
-            header_height = 90
-            # page_height = 892
-            # page_height_cm = 29.7
-            footer_height = 90
+    def calculate_headers_footers(self):
+        # header_offset = -50
+        self.header = 90
+        # page_height = 892
+        # page_height_cm = 29.7
+        self.footer = 90
 
+    def convert_write(self):
         print(f"==============CONVERT================")
-        if fmt == "html.flow":
-            fmt = "html"
-            flow = True
-        if not inpath:
-            raise ValueError(f"no input file given")
-        inpath = Path(inpath)
-        if not inpath.exists():
-            raise FileNotFoundError(f"input file does not exist: ({inpath})")
-        result = PDFArgs.convert_pdf(path=inpath, fmt=fmt, maxpages=maxpage)
+        self.process_args()
+        self.raw_html = PDFArgs.convert_pdf(path=self.inpath, fmt=self.out_fmt, maxpages=self.maxpage)
 
-        if flow:
-            tree = lxml.etree.parse(StringIO(result), lxml.etree.HTMLParser())
-            result_elem = tree.getroot()
-            HtmlUtil.add_ids(result_elem)
-            # this is slightly tacky
-            PDFUtil.remove_descendant_elements_by_tag("br", result_elem)
-            PDFUtil.remove_style(result_elem, [
-                "position",
-                # "left",
-                "border",
-                "writing-mode",
-                "width",  # this disables flowing text
-            ])
-            PDFUtil.remove_empty_elements(result_elem, ["span"])
-            PDFUtil.remove_empty_elements(result_elem, ["div"])
-            PDFUtil.remove_lh_line_numbers(result_elem)
-            PDFUtil.remove_large_fonted_elements(result_elem)
-            marker_xpath = ".//div[a[@name]]"
-            offset, pagesize, page_coords = PDFUtil.find_constant_coordinate_markers(result_elem, marker_xpath)
-            PDFUtil.remove_headers_and_footers(result_elem, pagesize, header_height, footer_height, marker_xpath)
-            PDFUtil.remove_style_attribute(result_elem, "top")
-            PDFUtil.remove_style(result_elem, ["left", "height"])
-            PDFUtil.remove_unwanteds(result_elem, unwanteds)
-            PDFUtil.remove_newlines(result_elem)
-            self.markup_parentheses(result_elem)
-            print(f"ref_counter {self.ref_counter}")
+        if self.flow:
+            self.html = self.tidy_flow()
+            assert len(self.html) > 0
+            print(f"flow {len(self.raw_html)}")
+        if self.outpath is None:
+            print(f"no outpath given")
+            return None
+        with open(str(self.outpath), "w") as f:
+            f.write(self.html)
+            print(f"wrote outpath {self.outpath}")
+        return self.outpath
 
-            HtmlTree.make_tree(result_elem, output_dir=outd, recs_by_section=RECS_BY_SECTION)
-
-            result = lxml.etree.tostring(result_elem).decode("UTF-8")
-            fmt = "flow.html"
-        if not outdir:
-            indir = Path(inpath).parent
-            outdir = indir
-            print(f"no outdir given, taking input {indir}")
-        if not outstem:
-            outstem = Path(inpath).stem
-        outfile = Path(outdir, f"{outstem}.{fmt}")
-        print(f"outfile {outfile}")
-        with open(str(outfile), "w") as f:
-            f.write(result)
-            print(f"wrote {f.name}")
+    def tidy_flow(self):
+        tree = lxml.etree.parse(StringIO(self.raw_html), lxml.etree.HTMLParser())
+        result_elem = tree.getroot()
+        HtmlUtil.add_ids(result_elem)
+        # this is slightly tacky
+        PDFUtil.remove_descendant_elements_by_tag("br", result_elem)
+        PDFUtil.remove_style(result_elem, [
+            "position",
+            # "left",
+            "border",
+            "writing-mode",
+            "width",  # this disables flowing text
+        ])
+        PDFUtil.remove_empty_elements(result_elem, ["span"])
+        PDFUtil.remove_empty_elements(result_elem, ["div"])
+        PDFUtil.remove_lh_line_numbers(result_elem)
+        PDFUtil.remove_large_fonted_elements(result_elem)
+        marker_xpath = ".//div[a[@name]]"
+        offset, pagesize, page_coords = PDFUtil.find_constant_coordinate_markers(result_elem, marker_xpath)
+        PDFUtil.remove_headers_and_footers(result_elem, pagesize, self.header, self.footer, marker_xpath)
+        PDFUtil.remove_style_attribute(result_elem, "top")
+        PDFUtil.remove_style(result_elem, ["left", "height"])
+        PDFUtil.remove_unwanteds(result_elem, self.unwanteds)
+        PDFUtil.remove_newlines(result_elem)
+        self.markup_parentheses(result_elem)
+        print(f"ref_counter {self.ref_counter}")
+        HtmlTree.make_sections_and_output(result_elem, output_dir=self.outdir, recs_by_section=RECS_BY_SECTION)
+        self.html = lxml.etree.tostring(result_elem).decode("UTF-8")
+        return self.html
 
     # class PDFArgs:
 
@@ -879,7 +1056,7 @@ class PDFArgs(AbstractArgs):
                 a = lxml.etree.SubElement(par, H_A)
                 for k, v in elem0.attrib.items():
                     a.attrib[k] = v
-                a.attrib[H_HREF] = "https://github.com/petermr/discussions"
+                a.attrib[A_HREF] = "https://github.com/petermr/discussions"
                 a.text = "([" + ref + "])"
                 current.addnext(a)
                 current = a
@@ -899,6 +1076,7 @@ class PDFDebug:
         self.max_curve = 10
         self.max_rect = 10
         self.image_coords_list = []
+        self.image_dict = dict()
 
     def debug_page_properties(self, page, debug=None, outdir=None):
         """debug print selected DEBUG_OPTIONS
@@ -965,11 +1143,20 @@ class PDFDebug:
         write_image = False
         resolution = 400  # may be better
             # see https://github.com/euske/pdfminer/blob/master/pdfminer/pdftypes.py
-        if n_image := len(page.images) > 0:
-            print(f"images {n_image}", end=" | ")
+        if (n_image := len(page.images)) > 0:
+            print(f"images {n_image}", end=" |\n")
             for i, image in enumerate(page.images[:maximage]):
-                print(f"image: {type(image)}: {image.values()}")
 
+                print(f"image: {type(image)}: {image.keys()} \n{image.values()}")
+                print(f"stream {image['stream']}")
+                print(f"xxyy {(image['x0'],image['x1']),(image['y0'],image['y1']),image['srcsize'],image['name'],image['page_number']}")
+                stream = image['stream']
+                width_height_bytes = ((image['srcsize']),image['stream']['Length'])
+                page_coords = (image['page_number'], (image['x0'],image['x1']),(image['y0'],image['y1']))
+                print(f"image:  {width_height_bytes} => {page_coords}")
+                if (width_height_bytes) in self.image_dict:
+                    print("clash: {(width_height_bytes)}")
+                self.image_dict[width_height_bytes] = page_coords
 
                 if not outdir:
                     pass
@@ -980,7 +1167,7 @@ class PDFDebug:
                     imagewriter.export_image(image)
                 page_height = page.height
                 image_bbox = (image[X0], page_height - image[Y1], image[X1], page_height - image[Y0])
-                print(f"image: {image_bbox}")
+                # print(f"image: {image_bbox}")
 
                 coord_stem = f"image_{page.page_number}_{i}_{self.format_bbox(image_bbox)}"
                 self.image_coords_list.append(coord_stem)
@@ -997,6 +1184,7 @@ class PDFDebug:
             #     for obj in p.layout:
             #         if isinstance(obj, LTImage):
             #             imagewriter.export_image(obj)
+        print(f"image_dict {self.image_dict}")
 
     def print_tables(self, page, odir=None):
         tables = page.find_tables()
@@ -1566,11 +1754,14 @@ def main(argv=None):
     python -m py4ami.ami_pdf \
         --inpath /Users/pm286/workspace/pyami/test/resources/ipcc/Chapter06/fulltext.pdf \
         --outdir /Users/pm286/workspace/pyami/temp/pdf/chap6/
+        --maxpage 100
+
     """
     print(f"running PDFArgs main")
     pdf_args = PDFArgs()
     try:
         pdf_args.parse_and_process()
+        pdf_args.convert_write()
     except Exception as e:
         print(traceback.format_exc())
         print(f"***Cannot run pyami***; see output for errors: {e} ")
