@@ -4,8 +4,8 @@ import sys
 import configparser
 import urllib.request
 from pathlib import Path
-
-from .file_lib import FileLib
+# local
+from py4ami.file_lib import FileLib
 
 
 class SymbolIni:
@@ -20,10 +20,13 @@ class SymbolIni:
     PYAMI_DIR = "PYAMI_DIR"
     PRIMITIVES = ["<class 'int'>", "<class 'bool'>", "<class 'float'>"]
     LOGDIR = "logs"  # maybe need to change this
+    SYMBOL_INI = "symbol.ini"
 
     SYMBOL_NOT_FOUND = "symbol not found"
+    SYM_START = "${"
+    SYM_END = "}"
 
-    logger = logging.getLogger("symbol.ini")
+    logger = logging.getLogger(SYMBOL_INI)
 
     def __init__(self, pyami):
         # FileLib.force_mkdir(self.LOGDIR)
@@ -31,9 +34,11 @@ class SymbolIni:
         self.symbols = None
         self.pyami = pyami
         pyami.symbol_ini = self
-
+        self.config = None
         self.setup_environment()
         self.process_config_files()
+        self.pyami_home = None
+        self.fileset = None
 
     def process_config_files(self):
         """ """
@@ -92,12 +97,14 @@ class SymbolIni:
             else:
                 self.logger.warning(f"*** cannot find config path {file} ***")
 
-    def get_home_dir(self, config_file):
+    @classmethod
+    def get_home_dir(cls, config_file):
         home = os.path.expanduser("~")
         file = home + config_file[len("~"):]
         return file
 
-    def get_code_dir(self, config_file):
+    @classmethod
+    def get_code_dir(cls, config_file):
         home = os.path.expanduser("~")
         file = __file__
         print("FILE ", file)
@@ -142,7 +149,7 @@ assumes value
             # we are going through ALL the symbols? possibly needs rethinking
             val = item[1]
             if val.startswith("http"):
-                if self.pyami.flagged(self.pyami.CHECK_URLS):
+                if self.pyami.is_flag_true(self.pyami.CHECK_URLS):
                     try:
                         with urllib.request.urlopen(val) as response:
                             html = response.read()
@@ -245,14 +252,12 @@ assumes value
 
         result = ""
         start = 0
-        SYM_START = "${"
-        SYM_END = "}"
         self.logger.info(f"expanding symbols in {arg}")
-        while SYM_START in arg[start:]:
-            idx0 = arg.index(SYM_START, start)
+        while SymbolIni.SYM_START in arg[start:]:
+            idx0 = arg.index(SymbolIni.SYM_START, start)
             result += arg[start:idx0]
-            idx1 = arg.index(SYM_END, start)
-            symbol = arg[idx0 + len(SYM_START):idx1]
+            idx1 = arg.index(SymbolIni.SYM_END, start)
+            symbol = arg[idx0 + len(SymbolIni.SYM_START):idx1]
             replace = self.symbols.get(symbol)
             if replace is None:
                 if not self.is_reserved_symbol(symbol):
@@ -266,9 +271,9 @@ assumes value
                 # symbol not found
                 raise ValueError(f"symbol replaces itself {symbol}")
             self.logger.debug(f"{idx0} {idx1} {symbol} {replace}")
-            orig = arg[idx0: idx1 + len(SYM_END)]
+            orig = arg[idx0: idx1 + len(SymbolIni.SYM_END)]
             result += replace if replace is not None else orig
-            start = idx1 + len(SYM_END)
+            start = idx1 + len(SymbolIni.SYM_END)
         result += arg[start:]
         if arg != result:
             self.logger.info(f"expanded {arg} to {result}")
@@ -276,7 +281,8 @@ assumes value
 
         # return arg[2:-1] if arg.startswith(SYM_START) and arg.endswith(SYM_END) else arg
 
-    def is_reserved_symbol(self, symbol):
+    @classmethod
+    def is_reserved_symbol(cls, symbol):
         """reserved symbols start with underscore"""
         return symbol is not None and symbol.startswith("_")
 
