@@ -324,61 +324,83 @@ class TestHtml(unittest.TestCase):
         """
         dictionary_file = Path(Resources.IPCC_CHAP06, "abbrev_as.xml")
         assert dictionary_file.exists(), f"file should exist {dictionary_file}"
-        ami_dict = AmiDictionary(dictionary_file)
+        ami_dict = AmiDictionary.create_from_xml_file(dictionary_file)
         ami_dict.ignorecase = False
-        target_path = Path(Resources.IPCC_CHAP06, "fulltext.flow.html")
+        inpath = Path(Resources.IPCC_CHAP06, "fulltext.flow.html")
         output_dir = Path(Resources.TEMP_DIR, "html")
         if not output_dir.exists():
             output_dir.mkdir()
         output_path = Path(output_dir, "chap6_index.html")
-        ami_dict.markup_html_from_dictionary(target_path, output_path, "pink")
+        ami_dict.markup_html_from_dictionary(inpath, output_path, "pink")
 
-    def test_markup_chapter_with_dictionary_css(self):
+    def test_markup_chapter_with_dictionary(self):
+        """read dictionary file and index a set of spans
+        Test: ami_dict.markup_html_from_dictionary
+        and add style
+        and write result]
+        TODO USE THIS!
+        """
+
+        dict_path = Path(Resources.IPCC_CHAP06, "abbrev_as.xml")
+        dict_path = Path(Resources.IPCC_CHAP06, "ipcc_ch6_rake.xml")
+        output_file = "chap6_marked.html"
+        output_file = "chap6_raked.html"
+        ami_dict = AmiDictionary.create_from_xml_file(dict_path, ignorecase = False)
+        input_path = Path(Resources.IPCC_CHAP06, "fulltext.flow.html")
+        # input_path = Path(Resources.IPCC_CHAP06, "chap6.flow.html")
+        print(f"reading pdf_html {input_path}")
+        html_output_dir = Path(Resources.TEMP_DIR, "html")
+        if not html_output_dir.exists():
+            html_output_dir.mkdir()
+        chap6_marked_path = Path(html_output_dir, output_file)
+
+
+        ami_dict.markup_html_from_dictionary(input_path, chap6_marked_path, "pink")
+        assert chap6_marked_path.exists(), f"marked-up html in {chap6_marked_path}"
+        with open(chap6_marked_path, "rb") as f:
+            marked_elem = lxml.etree.parse(f)
+
+
+
+    def test_extract_styles_as_css(self):
         """read dictionary file and index a set of spans
         Test: ami_dict.markup_html_from_dictionary
         and add style
         and write result
         """
-        ami_dict = AmiDictionary(Path(Resources.IPCC_CHAP06, "abbrev_as.xml"), ignorecase = False)
         target_path = Path(Resources.IPCC_CHAP06, "fulltext.flow.html")
         output_dir = Path(Resources.TEMP_DIR, "html")
         if not output_dir.exists():
             output_dir.mkdir()
-        output_path = Path(output_dir, "chap6_index.html")
+        output_path = Path(output_dir, "chap6_styled.html")
 
-        ami_dict.markup_html_from_dictionary(target_path, output_path, "pink")
-        with open(output_path, "rb") as f:
-            marked_elem = lxml.etree.parse(f)
-        styles = marked_elem.xpath(".//@style")
-        assert len(styles) == 316
+        with open(target_path, "rb") as f:
+            elem = lxml.etree.parse(f)
+        styles = elem.xpath(".//@style")
+        assert len(styles) >= 200
         style_set = set()
         for style in styles:
             style_set.add(style)
 
-        assert len(style_set) == 18
-        # for style in style_set:
-        #     print(f"style: {style}")
+        assert len(style_set) >= 10
 
         sorted_styles = sorted(style_set)
-        assert sorted_styles == ['',
-                                 'font-family: ArialMT; font-size: 10px;',
-                                 'font-family: Calibri-Bold; font-size: 10px;',
-                                 'font-family: Calibri-Bold; font-size: 12px;',
-                                 'font-family: Calibri-Bold; font-size: 13px;',
-                                 'font-family: Calibri; font-size: 10px;',
-                                 'font-family: Calibri; font-size: 10px; background-color: pink;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 11px;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 11px; background-color: pink;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 14px;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 15px;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 6px;',
-                                 'font-family: TimesNewRomanPS-BoldMT; font-size: 9px;',
-                                 'font-family: TimesNewRomanPS-ItalicMT; font-size: 11px;',
-                                 'font-family: TimesNewRomanPSMT; font-size: 11px;',
-                                 'font-family: TimesNewRomanPSMT; font-size: 11px; background-color: pink;',
-                                 'font-family: TimesNewRomanPSMT; font-size: 6px;',
-                                 'font-family: TimesNewRomanPSMT; font-size: 9px;'
-                                 ]
+        # assert sorted_styles == ['',
+        #                          'font-family: ArialMT; font-size: 10px;',
+        #                          'font-family: Calibri-Bold; font-size: 10px;',
+        #                          'font-family: Calibri-Bold; font-size: 12px;',
+        #                          'font-family: Calibri-Bold; font-size: 13px;',
+        #                          'font-family: Calibri; font-size: 10px;',
+        #                          'font-family: TimesNewRomanPS-BoldMT; font-size: 11px;',
+        #                          'font-family: TimesNewRomanPS-BoldMT; font-size: 14px;',
+        #                          'font-family: TimesNewRomanPS-BoldMT; font-size: 15px;',
+        #                          'font-family: TimesNewRomanPS-BoldMT; font-size: 6px;',
+        #                          'font-family: TimesNewRomanPS-BoldMT; font-size: 9px;',
+        #                          'font-family: TimesNewRomanPS-ItalicMT; font-size: 11px;',
+        #                          'font-family: TimesNewRomanPSMT; font-size: 11px;',
+        #                          'font-family: TimesNewRomanPSMT; font-size: 6px;',
+        #                          'font-family: TimesNewRomanPSMT; font-size: 9px;'
+        #                          ]
         css_classes = dict()
         for style in sorted_styles:
 
@@ -388,7 +410,25 @@ class TestHtml(unittest.TestCase):
                 css_style.extract_bold_italic_from_font_family()
 
 
-
+    def test_join_spans(self):
+        """join sibling spans with the same styles
+        """
+        html_path = Path(Resources.IPCC_CHAP04, "fulltext.flow.html")
+        html_element = lxml.etree.parse(str(html_path))
+        divs = html_element.xpath(".//div")
+        assert len(divs) == 3221
+        last_div = None
+        last_style = None
+        for div in divs:
+            spans = div.xpath("./span")
+            print(f"spans {len(spans)}")
+            for span in spans:
+                style = CSSStyle.create_css_style(span)
+                print(f"{style}")
+                if style == last_style:
+                    print(f"styles match")
+                last_span = span
+                last_style = style
 
 
     # ========================================
