@@ -7,16 +7,18 @@ import lxml.etree
 import lxml.html
 
 import os
+
+import test.test_all
+
 """NOTE REQUIRES LATEST pdfplumber"""
 import pdfplumber
 from PIL import Image
 # local
 from py4ami.ami_bib import Publication
-from py4ami.ami_html import AmiSpan
 
 from py4ami.ami_pdf import SVG_NS, SVGX_NS, CSSStyle, PDFArgs, PDFDebug
 from py4ami.ami_pdf import AmiPage, TextStyle, X, Y, SORT_XY, PDFImage
-from py4ami.ami_pdf import P_FONTNAME, P_HEIGHT, P_STROKING_COLOR, P_NON_STROKING_COLOR, P_TEXT, P_X0, P_Y0, P_X1
+from py4ami.ami_pdf import P_FONTNAME, P_HEIGHT, P_STROKING_COLOR, P_NON_STROKING_COLOR, P_TEXT, P_X0, P_Y0, P_X1, P_Y1
 from py4ami.ami_pdf import DEBUG_ALL, DEBUG_OPTIONS, WORDS, LINES, RECTS, CURVES, IMAGES, TABLES, HYPERLINKS, ANNOTS
 from py4ami.ami_html import HtmlUtil, STYLE,FILL, STROKE, FONT_FAMILY, FONT_SIZE
 from py4ami.ami_html import H_DIV, H_SPAN, H_A, H_B, H_BODY, H_P
@@ -44,11 +46,14 @@ IPCC_CHAP6_DIR = Path(IPCC_DIR, "Chapter06")
 IPCC_CHAP6_PDF = Path(IPCC_CHAP6_DIR, "fulltext.pdf")
 
 # arg_dict
-MAXPAGE = "maxpage"
+
 INDIR = "indir"
 INPATH = "inpath"
+MAXPAGE = "maxpage"
+PAGES = "pages"
 OUTDIR = "outdir"
 OUTFORM = "outform"
+OUTPATH = "outpath"
 OUTSTEM = "outstem"
 FLOW = "flow"
 # FORMAT = "fmt"
@@ -85,7 +90,8 @@ def make_html_dir():
     return html_dir
 
 
-class PDFTest(unittest.TestCase):
+class PDFTest(test.test_all.AmiAnyTest):
+
     MAX_PAGE = 5
     MAX_ITER = 20
 
@@ -245,93 +251,42 @@ class PDFTest(unittest.TestCase):
         args = f"--proj {proj} --apply svg2page"
         PyAMI().run_command(args)
 
-    @unittest.skip("Needs debugging")
+
+    @unittest.skip("page2sect NYI")
     def test_page2chap(self):
         proj = Resources.CLIMATE_10_PROJ_DIR
         args = f"--proj {proj} --apply page2sect"
         PyAMI().run_command(args)
 
-    def test_make_spans_from_charstream(self):
-        root = "pmc1421"
+    def test_make_ami_pages_with_spans_from_charstream_ipcc_chap6(self):
+        """The central AMI method to make HTML from PDF characters
+        TODO DEVELOP THIS
+        """
         root = "chap6"
         page_no = 0
-        page_no = 3
+        page_nos = range(3, 13)
         output_root = root
-        if root == "pmc1421":
-            input_pdf = PMC1421
-
-        elif root == "chap6":
-            input_pdf = Path(IPCC_CHAP6_PDF)
+        input_pdf = Path(IPCC_CHAP6_PDF)
         assert input_pdf.exists(), f"{input_pdf} should exist"
+        """y="793.815">Final Government Distribution  Chapter 6 IPCC WGIII AR6 </span></div>
+<div><span style="font-size: 11.04 px;font-family: TimesNewRomanPS-BoldMT;" y="55.719">D"""
+        bbox = BBox(xy_ranges=[[60, 999], [60, 790]])
 
-        with pdfplumber.open(input_pdf) as pdf:
-            page0 = pdf.pages[page_no]
-            print(f"crop: {page0.cropbox} media {page0.mediabox}, bbox {page0.bbox}")
-            print(f"rotation: {page0.rotation} doctop {page0.initial_doctop}")
-            print(f"width {page0.width} height {page0.height}")
-            print(f"text {page0.extract_text()[:2]}")
-            print(f"words {page0.extract_words()[:3]}")
+        output_dir = Path(Resources.TEMP_DIR, "pdf")
+        if not output_dir.exists():
+            print(f"output {output_dir}")
+            output_dir.mkdir()
+        for page_no in page_nos:
+            html = AmiPage.chars_to_spans(bbox, input_pdf, page_no)
 
-            print(f"char {page0.chars[:1]}")
-            span = None
-            span_list = []
-            maxchars = 999999
-            ndec_coord = 3 # decimals for coords
-            ndec_fontsize = 2
-            for ch in page0.chars[:maxchars]:
-                text_style = TextStyle()
-                text_style.set_font_family(ch.get(P_FONTNAME))
-                text_style.set_font_size(ch.get(P_HEIGHT), ndec=ndec_fontsize)
-                text_style.stroke = ch.get(P_STROKING_COLOR)
-                text_style.fill = ch.get(P_NON_STROKING_COLOR)
+            output_html = Path(output_dir, f"{output_root}_{page_no}.html")
+            with open(output_html, "wb") as f:
+                f.write(lxml.etree.tostring(html))
+                print(f" wrote html {output_html}")
+                assert output_html.exists()
 
-                x0 = round(ch.get(P_X0), ndec_coord)
-                x1 = round(ch.get(P_X1), ndec_coord)
-                y0 = round(ch.get(P_Y0), ndec_coord)
-                # style or y0 changes
-                if not span or not span.text_style or span.text_style != text_style or span.y0 != y0:
-                    if span:
-                        if span.text_style != text_style:
-                            print(f"{span.text_style.difference(text_style)} \n {span.string}")
-
-                        if span.y0 != y0:
-                            print(f""
-                                  f"Y {y0} != {span.y0}\n {span.string} {span.xx} ")
-                    span = AmiSpan()
-                    span_list.append(span)
-                    span.text_style = text_style
-                    span.y0 = y0
-                    span.x0 = x0 # set left x
-                span.x1 = x1 # update right x, including width
-                span.string += ch.get(P_TEXT)
-
-            top_div = lxml.etree.Element(H_DIV)
-            top_div.attrib["class"] = "top"
-            div = lxml.etree.SubElement(top_div, H_DIV)
-            last_span = None
-            for span in span_list:
-                if last_span is None or last_span.y0 != span.y0:
-                    div = lxml.etree.SubElement(top_div, H_DIV)
-                last_span = span
-                span.create_and_add_to(div)
-
-        for ch in page0.chars[:60000]:
-            col = ch.get('non_stroking_color')
-            if col:
-                print(f"txt {ch.get('text')} : col {col}")
-            if ch.get("size") > 20:
-                print(f"LARGE {ch}")
-        path = Path(Resources.TEMP_DIR, "pdf")
-        if not path.exists():
-            print(f"output {path}")
-            path.mkdir()
-        print(f"div {lxml.etree.tostring(div, encoding='UTF-8')}")
-        with open(Path(path, f"{output_root}_{page_no}.html"), "wb") as f:
-            f.write(lxml.etree.tostring(top_div))
-
-
-
-    def test_pdfplumber(self):
+    def test_pdfplumber_full_page_info(self):
+        """The definitive catalog of all objects on a page"""
         assert PMC1421.exists(), f"{PMC1421} should exist"
 
         # also ['_text', 'matrix', 'fontname', 'ncs', 'graphicstate', 'adv', 'upright', 'x0', 'y0', 'x1', 'y1',
@@ -357,8 +312,6 @@ class PDFTest(unittest.TestCase):
             assert first_page.cropbox == [0, 0, 595.22, 842]
             assert first_page.mediabox == [0, 0, 595.22, 842]
             assert first_page.bbox == (0, 0, 595.22, 842)
-            assert first_page.rotation == 0
-            assert first_page.initial_doctop == 0
             assert first_page.cached_properties == ['_rect_edges', '_edges', '_objects', '_layout']
             assert first_page.is_original
             assert first_page.pages is None
@@ -369,6 +322,49 @@ class PDFTest(unittest.TestCase):
             # rotate = 0 >
             assert first_page.annots == []
             assert first_page.hyperlinks == []
+            assert len(first_page.objects) == 2
+            assert type(first_page.objects) is dict
+            assert list(first_page.objects.keys()) == ['char', 'line']
+            assert len(first_page.objects['char']) == 4411
+            assert first_page.objects['char'][:2] == [
+                {'matrix': (9, 0, 0, 9, 319.74, 797.4203),
+                 'fontname': 'KAAHHD+Calibri,Italic',
+                 'adv': 0.319,
+                 'upright': True,
+                 'x0': 319.74, 'y0': 795.1703, 'x1': 322.611, 'y1': 804.1703,
+                 'width': 2.870999999999981, 'height': 9.0, 'size': 9.0,
+                 'object_type': 'char', 'page_number': 1,
+                 'text': 'J', 'stroking_color': None, 'non_stroking_color': (0.86667, 0.26667, 1, 0.15294),
+                 'top': 37.8297, 'bottom': 46.8297, 'doctop': 37.8297
+                 },
+                {'matrix': (9, 0, 0, 9, 322.6092, 797.4203), 'fontname': 'KAAHHD+Calibri,Italic', 'adv': 0.513, 'upright': True,
+                 'x0': 322.6092, 'y0': 795.1703, 'x1': 327.2262, 'y1': 804.1703, 'width': 4.617000000000019, 'height': 9.0, 'size': 9.0,
+                 'object_type': 'char', 'page_number': 1, 'text': 'o', 'stroking_color': None, 'non_stroking_color': (0.86667, 0.26667, 1, 0.15294),
+                 'top': 37.8297, 'bottom': 46.8297, 'doctop': 37.8297},
+                ],  f"first_page.objects['char'][0]  {first_page.objects['char'][0] }"
+            assert len(first_page.objects['line']) == 1, f" len(first_page.objects['line'])"
+            assert first_page.objects['line'][0] == {
+                 'bottom': 48.24000000000001,
+                 'doctop': 48.24000000000001,
+                 'evenodd': False,
+                 'fill': False,
+                 'height': 0.0,
+                 'linewidth': 1,
+                 'non_stroking_color': 0,
+                 'object_type': 'line',
+                 'page_number': 1,
+                 'pts': [(56.7, 793.76), (542.76, 793.76)],
+                 'stroke': True,
+                 'stroking_color': (0.3098, 0.24706, 0.2549, 0),
+                 'top': 48.24000000000001,
+                 'width': 486.06,
+                 'x0': 56.7,
+                 'x1': 542.76,
+                 'y0': 793.76,
+                 'y1': 793.76
+                                                     }, f"first_page.objects['line'][0]  {first_page.objects['line'][0] }"
+            # assert first_page.process_object(obj) == [], f"process_object (LTItem) {first_page.process_object()}"
+
             assert first_page.find_tables() == []
             assert first_page.extract_tables() == []
             assert first_page.extract_text()[:20] == "Journal of Medicine "
@@ -389,6 +385,7 @@ class PDFTest(unittest.TestCase):
 
             assert first_page.curves == []
             assert first_page.images == []
+            assert len(first_page.chars) == 4411 # same as first_page.objects['char'] I think
             assert first_page.chars[:1] == [
                 {'adv': 0.319,
                  'bottom': 46.8297,
@@ -411,6 +408,7 @@ class PDFTest(unittest.TestCase):
                  'y1': 804.1703
                  },
             ]
+            """first_page.objects['char']"""
             assert first_page.chars[0] == {'matrix': (9, 0, 0, 9, 319.74, 797.4203),
                                            'fontname': 'KAAHHD+Calibri,Italic', 'adv': 0.319,
                                            'upright': True, 'x0': 319.74, 'y0': 795.1703, 'x1': 322.611, 'y1': 804.1703,
@@ -471,8 +469,11 @@ class PDFTest(unittest.TestCase):
                 print(ch.get("text"), end="")
 
     def test_debug_page_properties_chap6(self):
-        """debug the PDF objects (crude)"""
-        maxpage = 20 # image is on page 8
+        """debug the PDF objects (crude)
+        outputs wordcount for page, and any image data.
+        Would ber better if we knew hoe to read PDFStream"""
+        maxpage = 9 # images on page 8, and 9
+        # maxpage = 9999 # image is on page 8
         outdir = Path(Resources.TEMP_DIR, "pdf", "chap6")
         pdf_debug = PDFDebug()
 
@@ -481,9 +482,48 @@ class PDFTest(unittest.TestCase):
             for page in pages[:maxpage]:
                 pdf_debug.debug_page_properties(page, debug=[WORDS, IMAGES], outdir=outdir)
         pdf_debug.write_summary(outdir=outdir)
+        print(f"pdf_debug {pdf_debug.image_dict}")
+        assert maxpage != 9 or pdf_debug.image_dict == {
+            ((1397, 779), 143448): (8, (72.0, 523.3), (412.99, 664.64)),
+            ((1466, 655), 122016): (8, (72.0, 523.3), (203.73, 405.38)),
+            ((1634, 854), 204349): (9, (80.9, 514.25), (543.43, 769.92))
+        }
+
+    @unittest.skip("LONG; other methods may be better")
+    def test_pdfminer_images(self):
+        import pdfminer
+        from pdfminer.image import ImageWriter
+        from pdfminer.high_level import extract_pages
+
+        path = Path(IPCC_CHAP6_PDF)
+        pages = list(extract_pages(path))
+        page = pages[10]
+
+        def get_image(layout_object):
+            if isinstance(layout_object, pdfminer.layout.LTImage):
+                print(f"LTImage {layout_object.__dir__()}")
+                return layout_object
+            if isinstance(layout_object, pdfminer.layout.LTContainer):
+                for child in layout_object:
+                    return get_image(child)
+            else:
+                return None
+
+        def save_images_from_page(page: pdfminer.layout.LTPage):
+            images = list(filter(bool, map(get_image, page)))
+            outdir = Path(Resources.TEMP_DIR, "pdf", "chap6", "pdf miner")
+            iw = ImageWriter(outdir)
+            for image in images:
+                iw.export_image(image)
+                print(f" image {image}")
+
+        save_images_from_page(page)
+
 
     def test_debug_page_properties(self):
-        """debug the PDF objects (crude)"""
+        """ high-level debug the PDF objects (crude) uses PDFDebug on 5-page document
+        finds WORDS (count) and IMAGE details
+        """
 
         outdir = Path(Resources.TEMP_DIR, "pdf", "pmc1421")
         if not outdir.exists():
@@ -501,14 +541,11 @@ class PDFTest(unittest.TestCase):
 
             for page in pages:
                 pdf_debug.debug_page_properties(page, debug=[WORDS, IMAGES])
-
-            # coord_list_file = Path(path, "image_coords.txt")
-            # basenames = [os.path.basename(pathx) for pathx in coord_paths]
-            # with open(coord_list_file, "w") as f:
-            #     f.write(f"{basenames}\n")
-            # print(f"wrote list of files based on coords: {coord_list_file}")
+            print(f"images: {pdf_debug.image_dict .keys()}")
 
     def test_bmp_png_to_png(self):
+        """convert bmp, jpgs, etc to PNG
+        """
         dirx = Path(Resources.IPCC_CHAP06, "image_bmp_jpg")
         outdir = Path(Resources.IPCC_CHAP06, "image")
         if not dirx.exists():
@@ -518,6 +555,8 @@ class PDFTest(unittest.TestCase):
         pdf_image.convert_all_suffixed_files_to_target(dirx, [".bmp", ".jpg"], ".png", outdir=outdir)
 
     def test_merge_pdf2txt_bmp_jpg_with_coords(self):
+        """creates coordinate data for images (20 pp doc) and also reads existing coord data from file
+        (? from AMI3-java or previous run) and tries to match them"""
         png_dir = Path(Resources.IPCC_CHAP06, "images")
         bmp_jpg_dir = Path(Resources.IPCC_CHAP06, "image_bmp_jpg")
         coord_file = Path(Resources.IPCC_CHAP06, "image_coords.txt")
@@ -567,7 +606,9 @@ class PDFTest(unittest.TestCase):
 
     # https://towardsdatascience.com/pdf-text-extraction-in-python-5b6ab9e92dd
 
-    def test_pdfminer_layout(self):
+    def test_pdfminer_debug_LTTextLine_LTTextBox_PMC1421(self):
+        """read PDF and chunk into text_lines and text_boxes
+        Keeps box coordinates but loses style"""
         from pdfminer.layout import LTTextLineHorizontal, LTTextBoxHorizontal
         # need to pass in laparams, otherwise pdfplumber page would not
         # have high level pdfminer layout objects, only LTChars.
@@ -575,16 +616,19 @@ class PDFTest(unittest.TestCase):
         page = pdf.pages[0].layout
         for element in page:
             if isinstance(element, LTTextLineHorizontal):
-                print(f"textlinehorizontal: ({element.bbox}): {element.get_text()}", end="")
+                # currently only seems to detect newline
+                print(f"textlinehorizontal: ({element.bbox}):{element.get_text()}:", end="")
             if isinstance(element, LTTextBoxHorizontal):
                 print(f">>start_text_box")
                 for text_line in element:
-                    print(f"dir: {text_line.__dir__()}")
-                    print(f"....textboxhorizontal: ({text_line.bbox}): {text_line.get_text()}", end="")
+                    # print(f"dir: {text_line.__dir__()}")
+                    # print(f"....textboxhorizontal: ({text_line.bbox}): {text_line.get_text()}", end="")
+                    pass
                 print(f"<<end_text_box")
 
     # https://stackoverflow.com/questions/34606382/pdfminer-extract-text-with-its-font-information
 
+    @unittest.skip("too much output")
     def test_pdfminer_font_and_character_output(self):
         """Examines every character and annotates it
         Typical:
@@ -669,14 +713,107 @@ LTPage
                 for page in pages[:max_page]:
                     pdf_debug.debug_page_properties(page, debug=options)
 
-    def test_make_structured_html(self):
-        """structures the flat HTML from pdfplumber"""
-        file = IPCC_CHAP6_PDF
-        assert file.exists(), f"chap6 {IPCC_CHAP6_PDF}"
+    def test_make_structured_html_MAIN(self):
+        """structures the flat HTML from pdfplumber, but no coordinates
+        Can still be used for word frequency, etc."""
+
+        print(f" converting {IPCC_CHAP6_PDF}")
+        assert IPCC_CHAP6_PDF.exists(), f"chap6 {IPCC_CHAP6_PDF}"
         pdf_args = PDFArgs()
-        pdf_args.arg_dict[INPATH] = file
+        pdf_args.arg_dict[INPATH] = IPCC_CHAP6_PDF
         pdf_args.arg_dict[MAXPAGE] = 10
+
         print(f"arg_dict {pdf_args.arg_dict}")
+        outfile = pdf_args.convert_write()
+        if not outfile:
+            print(f"no file written")
+        else:
+            print(f"check {outfile} exists")
+            assert outfile.exists(), f"outfile {outfile} should exist"
+
+
+    def test_make_structured_html_pages_MAIN(self):
+        """structures the flat HTML from pdfplumber, but no coordinates
+        Can still be used for word frequency, etc."""
+
+        print(f" converting {IPCC_CHAP6_PDF}")
+        assert IPCC_CHAP6_PDF.exists(), f"chap6 {IPCC_CHAP6_PDF}"
+        pdf_args = PDFArgs()
+        pdf_args.arg_dict[INPATH] = IPCC_CHAP6_PDF
+        # pdf_args.arg_dict[PAGES] = [(1,3), (5,10)]
+
+        print(f"arg_dict {pdf_args.arg_dict}")
+        outfile = pdf_args.convert_write()
+        if not outfile:
+            print(f"no file written")
+        else:
+            print(f"check {outfile} exists")
+            assert outfile.exists(), f"outfile {outfile} should exist"
+
+    @unittest.skip("not implemented - just to remind us")
+    def test_make_structured_html_cmdline_DEBUG(self):
+        """
+        Previous one gives:
+        arg_dict {'convert': 'html', 'flow': True, 'footer': 80, 'header': 80, 'indir': None,
+        'inpath': PosixPath('/Users/pm286/workspace/pyami/test/resources/ipcc/Chapter06/fulltext.pdf'),
+        'maxpage': 10, 'outdir': None, 'outform': 'html', 'outstem': 'fulltext'}
+
+        Tried to run:
+        python -m py4ami.ami_pdf
+        --inpath /Users/pm286/projects/semanticClimate/ipcc/ar6/wg3/Chapter02/fulltext.pdf
+        --maxpage 88 --outdir /Users/pm286/projects/semanticClimate/ipcc/ar6/wg3/Chapter02/
+
+        which didn't give flow html, but sections.
+        Its debug
+
+        Namespace(convert=None, debug=None, flow=True, footer=80, header=80, imagedir=None,
+        indir=None, inpath=['/Users/pm286/projects/semanticClimate/ipcc/ar6/wg3/Chapter02/fulltext.pdf'],
+         maxpage=[88], outdir=['/Users/pm286/projects/semanticClimate/ipcc/ar6/wg3/Chapter02/'],
+         outform='html', outstem='fulltext.flow', resolution=400, template=None)"""
+
+    def test_make_ipcc_html_spans(self):
+        """uses PDFMiner library (no coordinates I think)"""
+        sem_clim_dir = Path("/users/pm286", "projects", "semanticClimate")
+        if not sem_clim_dir.exists():
+            print(f"no ipcc dir {sem_clim_dir}, so skipping")
+            return
+        ipcc_dir = Path(sem_clim_dir, "ipcc", "ar6", "wg3")
+        assert ipcc_dir.exists(), f"ipcc_dir {ipcc_dir} does not exist"
+        chapter = "Chapter04"
+        print(f"Converting chapter: {chapter}")
+        pdf_args = PDFArgs()
+        chapter_dir = Path(ipcc_dir, chapter)
+        pdf_args.arg_dict[INDIR] = chapter_dir
+        assert pdf_args.arg_dict[INDIR].exists(), f"dir does not exist {chapter_dir}"
+        inpath = Path(chapter, "fulltext.pdf")
+        pdf_args.arg_dict[INPATH] = Path(chapter_dir, "fulltext.pdf")
+        assert pdf_args.arg_dict[INPATH].exists(), f"file does not exist {inpath}"
+        pdf_args.arg_dict[MAXPAGE] = 20
+        pdf_args.arg_dict[OUTFORM] = "flow.html"
+        outdir = Path(Resources.TEMP_DIR, "ipcc_html")
+        if not outdir.exists():
+            outdir.mkdir()
+        pdf_args.arg_dict[OUTDIR] = outdir
+        print(f"arg_dict {pdf_args.arg_dict}")
+
+        pdf_args.unwanteds = {
+            "chapter": {
+                "xpath": ".//div/span",
+                "regex": "^Chapter\\s+\\d+\\s*$"
+            },
+            "final_gov": {
+                "xpath": ".//div/span",
+                "regex": "^\\s*Final Government Distribution\\s*$"
+            },
+            "page": {
+                "xpath": ".//div/a",
+                "regex": "^\\s*Page\\s*\\d+\\s*$",
+            },
+            "wg3": {
+                "xpath": ".//div/span",
+                "regex": "^\\s*(IPCC AR6 WGIII)|(IPCC WGIII AR6)\\s*$",
+            },
+        }
         pdf_args.convert_write()
 
     @unittest.skip("too long")
@@ -733,6 +870,7 @@ LTPage
             }
             pdf_args.convert_write(unwanteds=unwanteds)
 
+
     def test_pdfminer_text_html_xml(self):
         # Use `pip3 install pdfminer.six` for python3
         """runs pdfinterpreter/converter over 5-page article and creates html and xml versions"""
@@ -810,25 +948,43 @@ LTPage
         assert "width" in css_style
         assert css_style.get("width") == "34"
 
-    def test_pdfminer_html(self):
+    def test_convert_pdf_to_html_and_save(self):
+        """Uses PDFArgs.convert_pdf to convert PDF to HTML and save
+        to temp
+        """
         # Use `pip3 install pdfminer.six` for python3
 
+        """reading py4ami/resources/projects/liion4/PMC4391421/fulltext.pdf"""
         pathx = Path(PMC1421)
 
+        # convert PDF to html
         result = PDFArgs.convert_pdf(
             path=pathx,
             fmt="html",
             caching=True,
         )
-        # print(f"result {result}")
+        # output dir
         html_dir = Path(Resources.TEMP_DIR, "html")
         if not html_dir.exists():
             html_dir.mkdir()
-        with open(Path(html_dir, "pmc4121.xml"), "w") as f:
+        # ouytput file
+        path = Path(html_dir, "pmc4121.xml")
+        # clean because text requires new file
+        if path.exists():
+            os.remove(path)
+        with open(path, "w") as f:
             f.write(result)
             print(f"wrote {f}")
+        assert path.exists(), f"should output html to {path}"
+        assert 76000 < os.path.getsize(path) < 77000, f"size should be in range , was {os.path.getsize(path)}"
+
+    def test_make_composite_lines_from_pdf_chap_6_3_toc(self):
+        path = Path(Resources.IPCC_CHAP06, "html", "chap6_3.html")
+        assert path.exists(), f"path exists {path}"
+        AmiPage.create_page_from_pdf_html(path)
 
     def test_main_help(self):
+        sys.argv=[]
         sys.argv.append("--help")
         try:
             main()
