@@ -7,8 +7,8 @@ import logging
 from lxml import etree, html
 import requests
 # local
-from py4ami.wikimedia import WikidataPage, ParserWrapper, WikidataExtractor, WikidataProperty
-from py4ami.ami_dict import AmiDictionary, WIKIDATA_ID, TERM, AMIDict
+from py4ami.wikimedia import WikidataPage, ParserWrapper, WikidataExtractor, WikidataProperty, WikidataFilter
+from py4ami.ami_dict import WIKIDATA_ID
 
 """This runs under Pycharm and also
 cd pyami # toplevel checkout
@@ -17,13 +17,13 @@ python3 -m test.test_wikidata
 
 try:
     from py4ami.wikimedia import WikidataLookup
-    from py4ami.ami_dict import AMIDict, AMIDictError, Entry, AmiDictionary
+    from py4ami.ami_dict import AMIDictError, AmiDictionary
 
     logging.info(f"loaded py4ami.ami_dict")
 except Exception:
     try:
         from py4ami.wikimedia import WikidataLookup
-        from py4ami.ami_dict import AMIDict, AMIDictError, Entry
+        from py4ami.ami_dict import AMIDictError
     except Exception as e:
         logging.error(f"Cannot import from py4ami.ami_dict")
 
@@ -215,8 +215,7 @@ class TestWikidataLookup(unittest.TestCase):
     def test_get_description(self):
         desc = WikidataPage("q407418").get_description()
         # assert desc == "chemical compound"
-        assert "compound" in desc # wikidata changed this!! 'organic compound used as flavouring and for analgesic properties'
-
+        assert "compound" in desc  # wikidata changed this!! 'organic compound used as flavouring and for analgesic properties'
 
     def test_attval_contains(self):
         """does a concatenated attavle contain a word
@@ -269,6 +268,7 @@ class TestWikidataLookup(unittest.TestCase):
             # selector = f".//div[contains(@class,'listview-item')]"
         print(f" selector {selector} ")
 
+        property_list = WikidataPage("q407418").root.xpath(selector)
 
         # <!-- property-[statement-list] container TOP LEVEL -->
         # <div class="wikibase-statementgroupview listview-item" id="P31" data-property-id="P31">
@@ -335,7 +335,7 @@ class TestWikidataLookup(unittest.TestCase):
 
         wikidata_page = WikidataPage("q407418")
         property_list = wikidata_page.get_data_property_list()
-        assert 74 > len(property_list) > 70
+        assert 78 >= len(property_list) >= 70
         assert wikidata_page.get_property_id_list()[:10] == [
             'P31', 'P279', 'P361', 'P117', 'P8224', 'P2067', 'P274', 'P233', 'P2017', 'P2054']
         assert wikidata_page.get_property_name_list()[:10] == [
@@ -375,7 +375,7 @@ class TestWikidataLookup(unittest.TestCase):
     <entry name="alpha-amyrenone" term="alpha-amyrenone"></entry> <!-- not in Wikidata -->
 </dictionary>        """
         path = Path(EO_COMPOUND_DIR, "dict_5.xml")
-        dictionary = AmiDictionary(str(path))
+        dictionary = AmiDictionary.create_from_xml_file(str(path))
         assert len(dictionary.entries) == 5
         entry = dictionary.get_entry("allylhexanoate")
         assert entry.get(WIKIDATA_ID) == "Q3270746"
@@ -385,9 +385,10 @@ class TestWikidataLookup(unittest.TestCase):
         dictionary.lookup_and_add_wikidata_to_entry(entry)
         assert entry.get(WIKIDATA_ID) == "Q27155908"
 
-        dictionary.write(Path(TEMP_DIR, "dict_5.xml"))
+        dictionary.write_to_file(Path(TEMP_DIR, "dict_5.xml"))
 
-    def test_add_wikidata_to_complete_dictionary(self):
+    @unittest.skip("LONG DOWNLOAD")
+    def test_add_wikidata_to_complete_dictionary_with_filter(self):
         """Takes existing dictionary and looks up Wikidata stuff for entries w/o WikidataID
         Need dictionary in AmiDictionary format"""
         input_dir = EO_COMPOUND_DIR
@@ -396,9 +397,10 @@ class TestWikidataLookup(unittest.TestCase):
         end_entry = 10
         input_path = Path(input_dir, "eoCompound1.xml")
         assert input_path.exists(), f"{input_path} should exist"
-        dictionary = AmiDictionary(str(input_path))
+        dictionary = AmiDictionary.create_from_xml_file(str(input_path))
         assert len(dictionary.entries) == 2114
         description = "chemical compound"
+
         for entry in dictionary.entries[start_entry: end_entry]:
             wikidata_id = AmiDictionary.get_wikidata_id(entry)
             if not AmiDictionary.is_valid_wikidata_id(wikidata_id):
@@ -411,7 +413,7 @@ class TestWikidataLookup(unittest.TestCase):
                     entry.attrib[WIKIDATA_ID] = AmiDictionary.NOT_FOUND
                 else:
                     print(f"found {wikidata_id} for {term} desc = {entry.get('desc')}")
-        dictionary.write(Path(output_dir, "eoCompound", "eoCompound1.xml"))
+        dictionary.write_to_file(Path(output_dir, "eoCompound", "eoCompound1.xml"))
 
     def test_disambiguation(self):
         """attempts to disambiguate the result of PMR-wikidata lookup"""
@@ -419,13 +421,13 @@ class TestWikidataLookup(unittest.TestCase):
         output_dir = TEMP_DIR
         input_path = Path(input_dir, "disambig.xml")
         assert input_path.exists(), f"{input_path} should exist"
-        dictionary = AmiDictionary(str(input_path))
+        dictionary = AmiDictionary.create_from_xml_file(str(input_path))
         assert len(dictionary.entries) == 9
         allowed_descriptions = {
             "chemical compound": "0.9",
             "group of isomers": "0.7",
         }
-        dictionary = AmiDictionary(str(input_path))
+        dictionary = AmiDictionary.create_from_xml_file(str(input_path))
         for entry in dictionary.entries:
             wikidata_id = AmiDictionary.get_wikidata_id(entry)
             if not AmiDictionary.is_valid_wikidata_id(wikidata_id):
@@ -438,7 +440,7 @@ class TestWikidataLookup(unittest.TestCase):
                     entry.attrib[WIKIDATA_ID] = AmiDictionary.NOT_FOUND
                 else:
                     print(f"found {wikidata_id} for {term} desc = {entry.get('desc')}")
-        dictionary.write(Path(output_dir, "eoCompound", "disambig.xml"))
+        dictionary.write_to_file(Path(output_dir, "eoCompound", "disambig.xml"))
 
     def test_get_instances(self):
         """<div class="wikibase-statementview-mainsnak-container">
@@ -458,6 +460,7 @@ class TestWikidataLookup(unittest.TestCase):
 </div>"""
         pass
 
+    @unittest.skip("LONG DOWNLOAD")
     def test_add_wikidata_to_imageanalysis_output(self):
         """creates dictionary from list of terms and looks up Wikidata"""
         terms = [
@@ -486,16 +489,12 @@ class TestWikidataLookup(unittest.TestCase):
             temp_dir.mkdir()
         # limit = 10000
         limit = 5
-        dictfile, amidict = AMIDict.create_from_list_of_strings_and_write_to_file(terms[:limit], title="compounds",
-                                                                                  wikidata=True, directory=temp_dir)
+        amidict, dictfile = AmiDictionary.create_dictionary_from_words(terms[:limit], title="compounds",
+                                                                                  wikidata=True, outdir=temp_dir)
         print(f"wrote to {dictfile}")
         assert os.path.exists(dictfile)
 
-    def test_json_wikidata_download(self):
-        """Test json/php api for downloadin wikidata
-        see https://github.com/sedthh/awena-wikidata-crawler
-        """
-        query = "Einstein"
+
 
     def test_wikidata_extractor(self):
         query = '2-fluorobenzoic acid'
@@ -504,7 +503,6 @@ class TestWikidataLookup(unittest.TestCase):
         id_dict = extractor.load(id)
         print(id_dict)
 
-
     def test_simple_wikidata_query(self):
         """get ID list for query results
         see https://www.wikidata.org/w/api.php for options
@@ -522,7 +520,7 @@ class TestWikidataLookup(unittest.TestCase):
     def test_wikidata_id_lookup(self):
         """test query wikidata by ID
         """
-        ids = "Q11966262" # "Dyschirius politus" a species of insect
+        ids = "Q11966262"  # "Dyschirius politus" a species of insect
         url_str = f"https://www.wikidata.org/w/api.php?" \
                   f"action=wbgetentities" \
                   f"&ids={ids}" \
@@ -531,7 +529,8 @@ class TestWikidataLookup(unittest.TestCase):
         response = requests.get(url_str)
         response_js = response.json()["entities"][ids]
         # print(f"pages for {ids}\n", pprint.pformat(response_js))
-        assert list(response_js.keys()) == ['pageid', 'ns', 'title', 'lastrevid', 'modified', 'type', 'id', 'labels', 'descriptions', 'aliases', 'claims','sitelinks']
+        assert list(response_js.keys()) == ['pageid', 'ns', 'title', 'lastrevid', 'modified', 'type', 'id', 'labels',
+                                            'descriptions', 'aliases', 'claims', 'sitelinks']
         assert response_js["id"] == ids
         assert response_js["title"] == "Q11966262"
         assert response_js["labels"]["en"]["value"] == "Dyschirius politus"
@@ -539,9 +538,9 @@ class TestWikidataLookup(unittest.TestCase):
         assert list(response_js["claims"].keys()) == [
             'P225', 'P105', 'P171', 'P31', 'P685', 'P846', 'P1939', 'P373', 'P815', 'P3151', 'P3186',
             'P3405', 'P2464', 'P1843', 'P7202', 'P7552', 'P6105', 'P6864', 'P8915', 'P3240', 'P2671',
-            'P3606','P8707','P10243'
+            'P3606', 'P8707', 'P10243'
         ]
-        ids = "P117" # chemical compound
+        ids = "P117"  # chemical compound
         url_str = f"https://www.wikidata.org/w/api.php?" \
                   f"action=wbgetentities" \
                   f"&ids={ids}" \
@@ -550,13 +549,17 @@ class TestWikidataLookup(unittest.TestCase):
         response = requests.get(url_str)
         response_js = response.json()["entities"][ids]
         # print(f"pages for {ids}\n", pprint.pformat(response_js))
-        assert list(response_js.keys()) == ['pageid', 'ns', 'title', 'lastrevid', 'modified', 'type', 'datatype', 'id', 'labels', 'descriptions', 'aliases', 'claims']
+        assert list(response_js.keys()) == ['pageid', 'ns', 'title', 'lastrevid', 'modified', 'type', 'datatype', 'id',
+                                            'labels', 'descriptions', 'aliases', 'claims']
         assert response_js["id"] == "P117"
         assert response_js["title"] == "Property:P117"
         assert response_js["labels"]["en"]["value"] == "chemical structure"
-        assert response_js["descriptions"]["en"]["value"] == "image of a representation of the structure for a chemical compound"
-        assert list(response_js["claims"].keys()) == ['P31', 'P1855', 'P3254', 'P2302', 'P1629', 'P1647', 'P2875', 'P1659']
-#        wikidata_page = WikidataPage.create_from_response(response)
+        assert response_js["descriptions"]["en"][
+                   "value"] == "image of a representation of the structure for a chemical compound"
+        assert list(response_js["claims"].keys()) == ['P31', 'P1855', 'P3254', 'P2302', 'P1629', 'P1647', 'P2875',
+                                                      'P1659']
+
+    #        wikidata_page = WikidataPage.create_from_response(response)
 
     def test_multiple_ids(self):
         ids = "P31|P117"
@@ -570,5 +573,14 @@ class TestWikidataLookup(unittest.TestCase):
         assert json_dict['entities']['P117']['labels']['en']['value'] == 'chemical structure'
         assert json_dict['entities']['P31']['labels']['en']['value'] == 'instance of'
 
+    def test_read_wikidata_filter(self):
+        path = Path(RESOURCES_DIR, "filter00.json")
+        filter = WikidataFilter.create_filter(path)
+        assert filter.json['plugh'] == "xyzzy"
+        assert filter.json['filter']['description'] == "chemical"
+        assert filter.json['filter'][
+                   'regex'] == "(chemical compound|chemical element)", f"found {filter.json['filter']['regex']}"
+
+
 if __name__ == '__main__':
-    unittest.main()
\ No newline at end of file
+    unittest.main()
